# -*- coding: utf-8 -*-

"""Console script for ec2_reaper."""

import click
import json
import logging
import botocore
import sys
from datetime import datetime

log = logging.getLogger()
log.setLevel(logging.INFO)
ch = logging.StreamHandler()
log.addHandler(ch)


try:
    import ec2_reaper
except botocore.exceptions.NoCredentialsError as e:
    log.error('boto3 was unable to find any AWS credentials. Please run `aws configure`')
    sys.exit(1)

def _is_py3():
    return sys.version_info >= (3, 0)

@click.command()
@click.argument('tagfilterstr', type=click.STRING, default=json.dumps(ec2_reaper.DEFAULT_TAG_MATCHER))
@click.option('--min-age', '-m', 'min_age', default=ec2_reaper.DEFAULT_MIN_AGE, type=click.INT,
    help='Instance must be (int)N seconds old before it will be considered for termination. Default: 300')
@click.option('--dry-run', '-d', 'dry_run', is_flag=True,
    help='Enable debug output and skip terminations.')
@click.option('--regions', '-r', type=click.STRING, multiple=True, default=ec2_reaper.DEFAULT_REGIONS,
    help='One or more regions to search. Searches all available regions by default.')
def main(tagfilterstr, min_age, dry_run, regions):
    """ec2-reaper [--min-age <seconds>] [--region region-1 --region region-2 ...] [--dry-run] <JSON filter expression>

    Terminate running instances matching tag requirements and a minimum age

    Default filter: [{"tag": "Name", "includes": [], "excludes": ["*"]}]

    Filter Behaviour:
    - An instance is reaped if tag value is in the include list
    - An instance is ignored if tag value is in the exclude list
    - Exclude overrides Include
    - `['*']` will match any value except empty for tag
    - Empty lists will match empty and missing tags.
        - eg: `{tag='Name', include=[], exclude=['*']}` will reap any instance without a 'Name' tag.
    - Instances must match *all* tag conditions if multiple are specified.
    - Stopped instances are always ignored.
    """
    if dry_run:
        log.setLevel(logging.DEBUG)
        log.warning('Dry Run mode enabled.')
        log.debug('Verbose output enabled.')
    logging.getLogger('botocore').setLevel(logging.WARNING)
    logging.getLogger('boto3').setLevel(logging.WARNING)

    tagfilter = json.loads(tagfilterstr)

    log.debug('Filter expression set: {}'.format(tagfilter))
    log.debug('Minimum age set to {} seconds'.format(min_age))
    if not regions:
        log.debug('Searching all available regions.')
    else:
        regions = list(regions) if isinstance(regions, tuple) else regions
        regions = regions if isinstance(regions, list) else [regions]
        regions = [r.decode('utf-8') if not _is_py3() and \
                   isinstance(r, unicode) else r for r in regions]
        log.debug('Searching the following regions: {}'.format(regions))

    log.info('Started ec2-reaper at {}'.format(datetime.now()))
    reaplog = ec2_reaper.reap(tagfilter, min_age=min_age, debug=dry_run, regions=regions)
    log.info('{} instances reaped out of {} found in {} regions.'.format(
        len([i for i in reaplog if i['reaped']]), len(reaplog),
        len(regions) if regions else 'all'))

    if len(reaplog) > 0:
        sys.exit(0)
    else:
        sys.exit(1)


if __name__ == "__main__":
    main()



from datetime import datetime, timedelta
import logging
import requests
import os
import sys
import json
import pytz

import ec2_reaper
from ec2_reaper import reap

def _is_py3():
    return sys.version_info >= (3, 0)

log = logging.getLogger(__name__)

DEFAULT_SLACK_ENDPOINT = ''

MIN_AGE = os.environ.get('MIN_AGE', ec2_reaper.DEFAULT_MIN_AGE)
REGIONS = os.environ.get('REGIONS', ec2_reaper.DEFAULT_REGIONS)
REGIONS = REGIONS.split(' ') if isinstance(REGIONS, str) else REGIONS

strclasses = str if _is_py3() else (str, unicode)
TAG_MATCHER = os.environ.get('TAG_MATCHER', ec2_reaper.DEFAULT_TAG_MATCHER)
TAG_MATCHER = json.loads(TAG_MATCHER) if isinstance(TAG_MATCHER, strclasses) else TAG_MATCHER

SLACK_ENDPOINT = os.environ.get('SLACK_ENDPOINT', None)

DEBUG = os.environ.get('DEBUG', False)
if DEBUG:
    log.setLevel(logging.DEBUG)
    logging.getLogger('botocore').setLevel(logging.INFO)
    logging.getLogger('boto3').setLevel(logging.INFO)
else:
    log.setLevel(logging.INFO)
    logging.getLogger('botocore').setLevel(logging.WARNING)
    logging.getLogger('boto3').setLevel(logging.WARNING)

# this is necessary because tz-aware dts aren't JSON serializable by default
class DateTimeJSONEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, datetime):
            return o.isoformat()
        return json.JSONEncoder.default(self, o)

def _respond(body, error=True, headers=None, status_code=500):
    o = {'statusCode': status_code, 'body': body}
    if headers:
        o['headers'] = headers
    return json.dumps(o, cls=DateTimeJSONEncoder)

def _get_expires(launch_time, min_age=MIN_AGE):
    # if launch_time is naive, assume UTC
    if launch_time.tzinfo is None or launch_time.tzinfo.utcoffset(launch_time) is None:
        launch_time = launch_time.replace(tzinfo=pytz.utc)
    min_age_dt = datetime.now() - timedelta(seconds=min_age)
    min_age_dt = min_age_dt.replace(tzinfo=ec2_reaper.LOCAL_TZ)
    log.info('Comparing launch_time {} to min_age_dt {}'.format(launch_time, min_age_dt))
    return (launch_time - min_age_dt).seconds


def _notify(msg, attachments=[]):
    if not SLACK_ENDPOINT:
        log.warning('Slack endpoint not configured!')
        return -1

    data = {'text': msg, 'attachements': attachments}
    headers = {'Content-Type': 'application/json'}
    r = requests.post(SLACK_ENDPOINT, json=data, headers=headers)

    if r.status_code != 200:
        log.error('Slack notification failed: (HTTP {}) {}'.format(r.status_code, r.text))

    return r.status_code


def handler(event, context):
    log.info("starting lambda_ec2_reaper at " + str(datetime.now()))

    log.debug('Filter expression set: {}'.format(TAG_MATCHER))
    log.debug('Minimum age set to {} seconds'.format(MIN_AGE))
    if not REGIONS:
        log.debug('Searching all available regions.')
    else:
        log.debug('Searching the following regions: {}'.format(REGIONS))

    reaperlog = reap(TAG_MATCHER, min_age=MIN_AGE, regions=REGIONS)

    # notify slack if anything was reaped
    reaped = [i for i in reaperlog if i['reaped']]
    log.info('{} instances reaped out of {} matched in {} regions.'.format(
        len(reaped), len(reaperlog), len(REGIONS) if REGIONS else 'all'))
    if len(reaped) > 0:
        msg = "The following instances have been terminated."
        attachments = []
        for i in reaped:
            attachments.append({
                'title': i['id'],
                'title_link': 'https://{region}.console.aws.amazon.com/ec2/v2/home?region={region}#Instances:search={id}'.format(id=i['id'], region=i['region']),
                'fields': [
                    {'title': 'Launch Time', 'value': i['launch_time'], 'short': True},
                    {'title': 'Region', 'value': i['region'], 'short': True},
                    {'title': 'Tags', 'value': i['tags'], 'short': True},
                ]
            })
        _notify(msg, attachments)

    # notify slack if anything matches but isn't old enough to be reaped
    too_young = [i for i in reaperlog if i['tag_match'] and not i['age_match'] and not i['reaped']]
    if len(too_young) > 0:
        log.info('{} instances out of {} matched were too young to reap.'.format(
            len(too_young), len(reaperlog)))

        msg = '@channel *WARNING*: The following instances are active but do not have tags. If they are left running untagged, they will be _*terminated*_.'
        attachments = []
        for i in too_young:
            attachments.append({
                'title': i['id'],
                'title_link': 'https://{region}.console.aws.amazon.com/ec2/v2/home?region={region}#Instances:search={id}'.format(id=i['id'], region=i['region']),
                'fields': [
                    {'title': 'Launch Time', 'value': i['launch_time'], 'short': True},
                    {'title': 'Expires In', 'short': True,
                        'value': "{} secs".format(_get_expires(i['launch_time']))},
                    {'title': 'Region', 'value': i['region'], 'short': True},
                    {'title': 'Tags', 'value': i['tags'], 'short': True},
                ]
            })
        _notify(msg, attachments)

    return _respond(reaperlog, error=False, status_code=200)



# -*- coding: utf-8 -*-
import boto3
import logging
import pytz
import os
import datetime
import time

"""EC2 Reaper"""

DEFAULT_TAG_MATCHER = [{"tag": "Name", "includes": [], "excludes": ["*"]}]
DEFAULT_MIN_AGE = 300
DEFAULT_REGIONS = None

# set up localtime for logging
LOCAL_TZ = pytz.timezone(time.tzname[time.localtime().tm_isdst])
LOCAL_TZ_NAME = LOCAL_TZ.tzname(datetime.datetime.now())

log = logging.getLogger()

def reap(tags=None, min_age=DEFAULT_MIN_AGE, regions=DEFAULT_REGIONS, debug=True):
    """reap - Terminate running instances matching tag requirements and a minimum age

    tags: List of dicts like `{'tag': 'sometag', include=['val1', ...], exclude=['val2', ...]}`
    min_age: Instance must be (int)N seconds old before it will be considered for termination. Default: 300
    regions: Stringy AWS region name or list of names to search. Searches all available regions by default.
    debug: If True, perform dry-run by skipping terminate API calls. Default: True

    Returns a list of dicts with instance that partially matches and their reap status.
    [{'id': i.id, 'tag_match': True, 'age_match': False, 'tags': i.tags, 'launch_time': i.launch_time, 'reaped': False, 'region': i.region}]

    Behaviour:
    - An instance is reaped if tag value is in the include list
    - An instance is ignored if tag value is in the exclude list
    - Exclude overrides Include
    - `['*']` will match any value except empty for tag
    - Empty lists will match empty and missing tags.
        - eg: `{tag='Name', include=[], exclude=['*']}` will reap any instance without a 'Name' tag.
    - Instances must match *all* tag conditions if multiple are specified.
    - Stopped instances are always ignored.
    """
    tags = tags if tags else DEFAULT_TAG_MATCHER
    regions = regions if regions else tuple(r['RegionName'] for r in boto3.client('ec2', region_name='us-east-1').describe_regions().get('Regions'))
    if isinstance(regions, str):
        regions = tuple(regions)

    old_log_level = None
    if debug and logging.getLevelName(log.level) != 'DEBUG':
        old_log_level = log.level
        log.setLevel(logging.DEBUG)

    reaperlog = []
    for region in regions:
        ec2 = boto3.resource('ec2', region_name=region)
        instances = [i for i in ec2.instances.filter(Filters=[{'Name': 'instance-state-name', 'Values': ['running']}])]
        log.debug('Found {} instances in {}'.format(len(instances), region))
        for i in instances:
            reaperlog_add = None

            local_time = i.launch_time.astimezone(LOCAL_TZ)
            log.debug('Checking {}, launched at {} with tags: {}'.format(
                i.id, local_time, i.tags))

            ct, ca = (_check_tags(i.tags, tags), _check_age(i.launch_time, min_age))

            if ct or ca:
                reaperlog_add = {'id': i.id, 'tag_match': ct, 'age_match': ca,
                                 'tags': i.tags, 'launch_time': i.launch_time,
                                 'reaped': False, 'region': region}

            if ct and not ca:
                log.warning('The following instance is a match, but isn\'t old enough yet: {} (launched at {} with tags: {})'.format(i.id, local_time, i.tags))

            if ct and ca:
                msg = '(NO-OP) ' if debug else ''
                msg += 'Reaping instance {} launched at {} {}'.format(i.id, local_time, LOCAL_TZ_NAME)
                log.warning(msg)
                if not debug:
                    i.terminate()
                reaperlog_add['reaped'] = True

            if reaperlog_add:
                reaperlog.append(reaperlog_add)

    if old_log_level:
        log.setLevel(old_log_level)

    return reaperlog

def _check_tags(instance_tags, matching_tags):
    log.debug('Comparing instance tags {} to filters {}'.format(
    instance_tags, matching_tags))
    instance_tags = instance_tags if instance_tags else []
    for mt in matching_tags:

        # reap if `tag` is not present and excludes == ['*']
        # eg: {tag: Name, excludes: ['*']}. If it has a Name, don't reap it.
        instance_tag_keys = [i['Key'] for i in instance_tags]

        log.debug('excludes: {} (matches? {})'.format(
            mt.get('excludes'), (mt.get('excludes') == [u'*'])))
        log.debug('tag: {} (in instance tags? {})'.format(
            mt.get('tag'), mt.get('tag') in instance_tag_keys))
        if mt.get('excludes') == [u'*'] and mt.get('tag') not in instance_tag_keys:
            log.debug('Tag "{}" not present; instance is reapable.'.format(mt.get('tag')))
            return True

        # check instance tags
        for it in instance_tags:
            if it.get('Key') != mt.get('tag'):
                continue

            # matching reapme tag exists and is not empty
            if mt.get('includes') == [u'*'] and it.get('Value'):
                log.debug('Tag "{}" is present and `include` is wildcarded; instance is reapable.'.format(mt.get('tag')))
                return True

            # tag exists, matches a value in includes, doesn't match any excludes
            if it.get('Value') in mt.get('includes') and it.get('Value') not in mt.get('excludes'):
                log.debug('Tag "{}" present, its value is in `includes` and not in `excludes`; instance is reapable.'.format(mt.get('tag')))
                return True
    return False

def _check_age(launch_time, min_age):
    # get local timezone info to compare against utc
    min_age_dt = datetime.datetime.now() - datetime.timedelta(seconds=min_age)
    min_age_dt = min_age_dt.replace(tzinfo=LOCAL_TZ)
    if launch_time < min_age_dt:
        log.debug('')
        return True
    else:
        return False



# -*- coding: utf-8 -*-

"""Top-level package for EC2 Reaper."""

import sys

__author__ = """Shaun Martin"""
__email__ = 'shaun@samsite.ca'
__version__ = '0.1.4'

if sys.version_info >= (3, 0):
    from ec2_reaper.ec2_reaper import reap
    from ec2_reaper.ec2_reaper import DEFAULT_TAG_MATCHER
    from ec2_reaper.ec2_reaper import DEFAULT_MIN_AGE
    from ec2_reaper.ec2_reaper import DEFAULT_REGIONS
    from ec2_reaper.ec2_reaper import LOCAL_TZ
    from ec2_reaper import aws_lambda
else:
    from ec2_reaper import reap
    from ec2_reaper import DEFAULT_TAG_MATCHER
    from ec2_reaper import DEFAULT_MIN_AGE
    from ec2_reaper import DEFAULT_REGIONS
    from ec2_reaper import LOCAL_TZ
    import aws_lambda



import logging
import json
import sys
from datetime import datetime, timedelta

from ec2_reaper import aws_lambda
from ec2_reaper import LOCAL_TZ

logging.basicConfig(level=logging.DEBUG)
logging.getLogger('botocore').setLevel(logging.INFO)
logging.getLogger('boto3').setLevel(logging.INFO)

if sys.version_info >= (3, 0):
    from unittest.mock import patch
else:
    from mock import patch

# when no results, handler should have called reap, *not* called (slack) notify,
# and should have returned a happy response json obj,
@patch.object(aws_lambda, 'reap')
@patch.object(aws_lambda, '_notify')
def test_reap_no_results(mock_notify, mock_reap):
    mock_reap.return_value = []
    r = json.loads(aws_lambda.handler({}, {}))

    mock_notify.assert_not_called()
    mock_reap.assert_called_once()
    assert r['statusCode'] == 200
    assert r['body'] == []

# with pos and neg results, handler should have called reap,
# called (slack) notify, and should have returned a happy response json obj with
# a body which contains all found instances and N reaped and N not reaped
@patch.object(aws_lambda, 'reap')
@patch.object(aws_lambda, '_notify')
def test_reap_2neg_1pos(mock_notify, mock_reap):
    match_time = datetime.now() - timedelta(seconds=500)
    nomatch_time = datetime.now()
    mock_reap_results =  [
        {'id': 'i-11111111', 'tag_match': True, 'age_match': False, 'tags': [],
         'launch_time': nomatch_time, 'reaped': False, 'region': 'us-east-1'},
        {'id': 'i-22222222', 'tag_match': False, 'age_match': True,
         'tags': [{'Key': 'Name', 'Value': 'somename'}],
         'launch_time': match_time, 'reaped': False, 'region': 'us-east-1'},
        {'id': 'i-11111111', 'tag_match': True, 'age_match': True, 'tags': [],
         'launch_time': match_time, 'reaped': True, 'region': 'us-east-1'},
    ]
    mock_reap.return_value = mock_reap_results
    r = json.loads(aws_lambda.handler({}, {}))

    mock_notify.assert_called()
    mock_reap.assert_called_once()
    assert r['statusCode'] == 200
    assert r['body'] == json.loads(json.dumps(mock_reap_results, cls=aws_lambda.DateTimeJSONEncoder))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `ec2_reaper` package."""

import pytest
import boto3
import sys
import time
from datetime import datetime, timedelta

from moto import mock_ec2
from click.testing import CliRunner

from ec2_reaper import ec2_reaper
from ec2_reaper import cli



import logging
log = logging.getLogger()
log.setLevel(logging.DEBUG)

logging.getLogger('boto3').setLevel(logging.CRITICAL)
logging.getLogger('botocore').setLevel(logging.CRITICAL)

@pytest.fixture
def response():
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """
    # import requests
    # return requests.get('https://github.com/audreyr/cookiecutter-pypackage')


def test_content(response):
    """Sample pytest test function with the pytest fixture as an argument."""
    # from bs4 import BeautifulSoup
    # assert 'GitHub' in BeautifulSoup(response.content).title.string

@mock_ec2
def _launch_instances(tags=None):
    '''
    tags must be in `[{'Key':'somekey', 'Value': 'someval'}, ...]` format
    '''
    ami_id = 'ami-1234abcd'
    client = boto3.client('ec2', region_name='us-west-2')
    params = {'ImageId': ami_id, 'MinCount': 1, 'MaxCount': 1}
    if tags:
        params['TagSpecifications'] = [{'ResourceType': 'instance', 'Tags': tags}]
    instance = client.run_instances(**params)['Instances'][0]

    instance['State']['Code'] = 16
    instance['State']['Name'] = 'running'

    instance = boto3.resource('ec2', region_name='us-west-2').Instance(instance[u'InstanceId'])
    if tags:
        log.debug('creating tags: {}'.format(tags))
        instance.create_tags(Tags=tags)

    log.debug('launched instance: {}'.format(instance))


def test_cli_help():
    """Test the CLI Help."""
    runner = CliRunner()
    help_result = runner.invoke(cli.main, ['--help'])
    assert help_result.exit_code == 0
    assert 'Show this message and exit.' in help_result.output

@mock_ec2
def test_cli_min_age():
    ### test with mock instances and defaults
    runner = CliRunner()
    _launch_instances()
    time.sleep(6)
    result = runner.invoke(cli.main, ['-d', '--min-age', '5'])
    assert result.exit_code == 0

@mock_ec2
def test_cli_tagstr():
    ### test with mock instances and defaults
    runner = CliRunner()
    _launch_instances(tags=[{'Key': 'Name', 'Value': 'somename'}])
    time.sleep(6)
    result = runner.invoke(cli.main, ['-d', '--min-age', '5', '[{"tag": "Name", "includes": ["somename"], "excludes": []}]'])
    assert result.exit_code == 0

@mock_ec2
def test_cli_allregions():
    """Test the CLI Help."""
    runner = CliRunner()
    runner = CliRunner()
    _launch_instances(tags=[{'Key': 'Name', 'Value': 'somename'}])
    time.sleep(6)
    result = runner.invoke(cli.main, ['-d', '--min-age', '5'])
    assert result.exit_code == 0

@mock_ec2
def test_cli_oneregion():
    """Test the CLI Help."""
    runner = CliRunner()
    _launch_instances(tags=[{'Key': 'Name', 'Value': 'somename'}])
    time.sleep(6)
    # instance launches into us-west-2
    result = runner.invoke(cli.main, ['-d', '--min-age', '5', '-r', 'us-east-1'])
    assert result.exit_code > 0

@mock_ec2
def test_cli_tworegions():
    """Test the CLI Help."""
    runner = CliRunner()
    _launch_instances(tags=[{'Key': 'Name', 'Value': 'somename'}])
    time.sleep(6)
    result = runner.invoke(cli.main, ['-d', '--min-age', '5', '-r', 'us-east-1', '-r', 'us-west-2'])
    assert result.exit_code == 0

@mock_ec2
def test_nomatch_tag_nomatch_age():
    ### test with mock instances and defaults
    _launch_instances(tags=[{'Key':'Name', 'Value': 'somename'}])
    reaperlog = ec2_reaper.reap()
    assert len(reaperlog) == 0

@mock_ec2
def test_match_tag_nomatch_age():
    ### test with mock instances and defaults
    _launch_instances()
    reaperlog = ec2_reaper.reap()
    assert len(reaperlog) == 1
    assert reaperlog[0]['tag_match']
    assert not reaperlog[0]['age_match']
    assert not reaperlog[0]['reaped']

@mock_ec2
def test_match_tag_includes():
    ### test with mock instances and defaults
    _launch_instances(tags=[{'Key':'Name', 'Value': 'somename'}])
    reaperlog = ec2_reaper.reap(tags=[{'tag':'Name', 'includes': ['somename'], 'excludes': ['*']}])
    assert len(reaperlog) == 1
    assert reaperlog[0]['tag_match']
    assert not reaperlog[0]['age_match']
    assert not reaperlog[0]['reaped']

@mock_ec2
def test_nomatch_tag_excludes():
    ### test with mock instances and defaults
    _launch_instances(tags=[{'Key':'Name', 'Value': 'somename'}])
    time.sleep(6)
    reaperlog = ec2_reaper.reap(min_age=5, tags=[{'tag':'Name',
                                                  'includes': ['nothingotmatch'],
                                                  'excludes': ['somename']}])
    assert len(reaperlog) == 1
    assert not reaperlog[0]['tag_match']
    assert reaperlog[0]['age_match']
    assert not reaperlog[0]['reaped']

@mock_ec2
def test_nomatch_tag_match_age():
    ### test with mock instances and defaults
    _launch_instances(tags=[{'Key':'Name', 'Value': 'somename'}])
    time.sleep(6)
    reaperlog = ec2_reaper.reap(min_age=5)
    assert len(reaperlog) == 1
    assert not reaperlog[0]['tag_match']
    assert reaperlog[0]['age_match']
    assert not reaperlog[0]['reaped']

@mock_ec2
def test_match_tag_match_age():
    ### test with mock instances and defaults
    _launch_instances()
    time.sleep(6)
    reaperlog = ec2_reaper.reap(min_age=5)
    assert len(reaperlog) == 1
    assert reaperlog[0]['tag_match']
    assert reaperlog[0]['age_match']
    assert reaperlog[0]['reaped']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('VERSION') as version_file:
    version = version_file.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup_requirements = [
    'pytest-runner',
    # TODO(inhumantsar): put setup requirements (distutils extensions, etc.) here
]

with open('requirements_dev.txt') as f:
    requirements_dev = f.read().splitlines()


setup(
    name='ec2-reaper',
    version=version,
    description="CLI & module for terminating instances that match tag and age requirements.",
    long_description=readme + '\n\n' + history,
    author="Shaun Martin",
    author_email='shaun@samsite.ca',
    url='https://github.com/inhumantsar/python-ec2-reaper',
    packages=['ec2_reaper'],
    entry_points={
        'console_scripts': [
            'ec2-reaper=ec2_reaper.cli:main'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license="BSD license",
    zip_safe=False,
    keywords='ec2-reaper',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    tests_require=requirements_dev+requirements,
    setup_requires=setup_requirements,
)
